<?php declare(strict_types=1);

/**
 * Debitor controller for Sale Documents
 *
 * @package     Sales
 * @subpackage  Controller
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Paul Mehrer <p.mehrer@metaways.de>
 * @copyright   Copyright (c) 2023-2024 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

/**
 * Address controller for Sale Documents
 *
 * @package     Sales
 * @subpackage  Controller
 */
class Sales_Controller_Document_Debitor extends Tinebase_Controller_Record_Abstract
{
    use Tinebase_Controller_SingletonTrait;

    /**
     * the constructor
     *
     * don't use the constructor. use the singleton
     */
    protected function __construct()
    {
        $this->_applicationName = Sales_Config::APP_NAME;
        $this->_backend = new Tinebase_Backend_Sql([
            Tinebase_Backend_Sql_Abstract::MODEL_NAME => Sales_Model_Document_Debitor::class,
            Tinebase_Backend_Sql_Abstract::TABLE_NAME => Sales_Model_Document_Debitor::TABLE_NAME,
            Tinebase_Backend_Sql_Abstract::MODLOG_ACTIVE => true,
        ]);
        $this->_modelName = Sales_Model_Document_Debitor::class;
        $this->_purgeRecords = false;
        $this->_doContainerACLChecks = false;
    }

    protected function _checkGrant($_record, $_action, $_throw = TRUE, $_errorMessage = 'No Permission.', $_oldRecord = null)
    {
        if ($_action === self::ACTION_GET) {
            $_action = Sales_Model_DivisionGrants::GRANT_READ_DEBITOR;
        } else {
            $_action = Sales_Model_DivisionGrants::GRANT_ADMIN_DEBITOR;
        }

        return parent::_checkGrant($_record, $_action, $_throw, $_errorMessage, $_oldRecord); // TODO: Change the autogenerated stub
    }
}
